import numpy as np


class Problem(object):
    def __init__(self, n_vars, iconstr_norm, is_sat, description_str):
        self.n_vars = n_vars
        self.n_lits = 2 * n_vars
        self.n_cons = len(iconstr_norm)
        self.n_cells = self.compute_n_cells(iconstr_norm)
        self.is_sat = is_sat
        self.description_str = description_str
        self.edges_list, self.edges_weight_list = self.compute_edges_list(iconstr_norm)

    def compute_n_cells(self, iconstr_norm):
        n_cells = 0
        for icon in iconstr_norm: n_cells += len(icon)
        return n_cells

    # Each row in edges_list is [vlit, iconstraint, weight]
    # var  x[0]  x[1]  ... x[n-1]  ~x[0]  ~x[1]  ... ~x[n-1]
    # vlit   0     1    ...  n-1      n     n+1  ...   2n-1
    def compute_edges_list(self, iconstr_norm):
        edges_list = np.zeros([self.n_cells, 2], dtype=np.int)
        edges_weight_list = np.zeros(self.n_cells, dtype=np.float)
        cell = 0
        for icon_id, icon in enumerate(iconstr_norm):
            # variable-constraint link
            for var, ratio in icon:
                vlit = (var + self.n_vars if ratio < 0 else var)
                edges_list[cell, :] = [vlit, icon_id]
                edges_weight_list[cell] = abs(ratio)
                cell += 1
        assert(cell == self.n_cells)
        return edges_list, edges_weight_list


# TODO Batch Learning
def shift_vars(item, offset):
    return tuple([item[0] + offset, item[1]])


def shift_iconstraints(iconstr_norm, offset):
    return [[shift_vars(item, offset) for item in icon]
            for icon in iconstr_norm]


def mk_batch_problem(problems):
    all_description_str = []
    all_iconstraints = []
    all_is_sat = []
    offset = 0
    for description_str, n_vars, iconstr_norm, is_sat in problems:
        all_description_str.append(description_str)
        all_iconstraints.extend(shift_iconstraints(iconstr_norm, offset))
        all_is_sat.append(is_sat)
        offset += n_vars
    return Problem(offset, all_iconstraints, all_is_sat, all_description_str)


# if __name__ == "__main__":
#     problems = [
#         ("prob1", 3, [[(0, 2), (1, 3)], [(0, 1), (1, 1), (2, -1)], [(1, 1), (2, 2)]], [5, 1, 3], True),
#         ("prob2", 3, [[(0, 2), (1, 3)], [(0, 1), (1, 1), (2, -1)], [(1, 1), (2, 2)]], [5, 2, 3], False),
#         ("prob3", 3, [[(0, 2), (1, 3)], [(0, 1), (1, 1), (2, -1)], [(2, 2), (2, 3)]], [5, 1, 4], True)
#     ]
#     batch_problem = mk_batch_problem(problems)
#     print('n_vars:', batch_problem.n_vars)
#     print('n_cells:', batch_problem.n_cells)
#     print('n_cons:', batch_problem.n_cons)
