import cplex


def solve_ilp_opt(n_vars, weight, edges):  # modify for mis problem

    # print("DBG", n_vars, weight, edges)

    cpx = cplex.Cplex()
    cpx.set_results_stream(None)
    cpx.set_log_stream(None)

    # variables
    cpx.variables.add(names=["x" + str(i) for i in range(n_vars)], obj=weight)
    for i in range(n_vars):
        cpx.variables.set_types(i, cpx.variables.type.integer)
        cpx.variables.set_lower_bounds(i, 0)
        cpx.variables.set_upper_bounds(i, 1)

    # constraints
    assert(len(weight) == n_vars)

    for _idx, edge in enumerate(edges):
        ind_list = []
        val_list = []
        for i in edge:
            ind_list.append("x" + str(i))
            val_list.append(1)
        cpx.linear_constraints.add(
            lin_expr=[cplex.SparsePair(ind=ind_list, val=val_list)],
            rhs=[1],
            names=["c" + str(_idx)],
            senses=["L"]
        )

    # objective function
    cpx.objective.set_sense(cpx.objective.sense.maximize)

    # solving
    cpx.solve()
    return int(cpx.solution.get_objective_value())


def solve_ilp_sat(n_vars, iconstraints_left, iconstraints_right):  # not used

    # print("DBG", n_vars, iconstraints_left, iconstraints_right)
    cpx = cplex.Cplex()
    cpx.set_results_stream(None)
    cpx.set_log_stream(None)

    # variables
    cpx.variables.add(names=["x" + str(i) for i in range(n_vars)])
    for i in range(n_vars):
        cpx.variables.set_types(i, cpx.variables.type.integer)
        cpx.variables.set_lower_bounds(i, 0)
        cpx.variables.set_upper_bounds(i, 1)

    # constraints
    assert(len(iconstraints_left) == len(iconstraints_right))
    for i in range(len(iconstraints_left)):
        ind_list = []
        val_list = []
        for _ind, _val in iconstraints_left[i]:
            ind_list.append("x" + str(_ind))
            val_list.append(_val)
        cpx.linear_constraints.add(
            lin_expr=[cplex.SparsePair(ind=ind_list, val=val_list)],
            rhs=[iconstraints_right[i]],
            names=["c" + str(i)],
            senses=["G"]
        )

    # objective function
    # ...
    
    # solving
    cpx.solve()
    return cpx.solution.is_primal_feasible()


# Example:
#
# num_vars = 5
# weight = [6, 10, 5, 1, 8]
# edges = [[0,1], [0,3], [1,2], [2,3], [3,4]]
# print(solve_ilp_opt(num_vars, weight, edges))
