import pickle
import os
import time
from tqdm import tqdm

import torch
import torch.nn as nn
import torch.optim as optim

from neuroilp import NeuroILP
import argparse

if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('--dim', type=int, default=128)
    parser.add_argument('--n_rounds', type=int, default=26)
    parser.add_argument('--model_file', type=str, default=None)
    parser.add_argument('--data_file', type=str, default=None)
    parser.add_argument('--log_file', type=str, default=None)
    args = parser.parse_args()

    project_dir = '/pub/data/liumh/neuro_pb_kp/'
    log_file = open(os.path.join(project_dir, 'log', args.log_file), 'w')

    net = NeuroILP(args)
    net = net.cuda()
    loss_fn = nn.BCELoss()
    # optim = optim.Adam(net.parameters(), lr=0.00002, weight_decay=1e-10)
    sigmoid = nn.Sigmoid()

    if args.model_file is not None:
        print('Restore model from', args.model_file)
        model = torch.load(os.path.join(project_dir, 'model', args.model_file))
        # start_epoch = model['epoch']
        # best_acc = model['acc']
        net.load_state_dict(model['state_dict'])
    else:
        print('Error: Model loading failed.')
        exit(1)

    with open(os.path.join(project_dir, 'data/test_nobatch', args.data_file), 'rb') as f:
        test = pickle.load(f)
        print('Load test data from', args.data_file)

    test_bar = tqdm(test)
    TP, TN, FN, FP = torch.zeros(1).long(), torch.zeros(1).long(), torch.zeros(1).long(), torch.zeros(1).long()
    loss_sum, loss_cnt = 0, 0
    net.eval()
    time_st = time.perf_counter()
    for _, prob in enumerate(test_bar):
        # optim.zero_grad()
        if _ == 0:
            continue
        outputs = net(prob)
        target = torch.Tensor(prob.is_sat).cuda().float()
        # print(outputs.shape, target.shape)
        # print(outputs, target)
        outputs = sigmoid(outputs)
        loss = loss_fn(outputs, target)
        loss_sum += loss.item()
        loss_cnt += 1
        preds = torch.where(outputs > 0.5, torch.ones(outputs.shape).cuda(), torch.zeros(outputs.shape).cuda())

        TP += (preds.eq(1) & target.eq(1)).cpu().sum()
        TN += (preds.eq(0) & target.eq(0)).cpu().sum()
        FN += (preds.eq(0) & target.eq(1)).cpu().sum()
        FP += (preds.eq(1) & target.eq(0)).cpu().sum()
        TOT = TP + TN + FN + FP

        desc = 'acc: %.3f, TP: %.3f, TN: %.3f, FN: %.3f, FP: %.3f' % (
            (TP.item() + TN.item()) * 1.0 / TOT.item(),
            TP.item() * 1.0 / TOT.item(),
            TN.item() * 1.0 / TOT.item(),
            FN.item() * 1.0 / TOT.item(),
            FP.item() * 1.0 / TOT.item())
        if (_ + 1) % 100 == 0:
            print(desc, file=log_file, flush=True)
    print(desc, file=log_file, flush=True)
    time_ed = time.perf_counter()

    acc = (TP.item() + TN.item()) * 1.0 / TOT.item()
    loss = loss_sum / loss_cnt
    print('final acc:', acc, file=log_file)
    print('average loss:', loss, file=log_file)
    print('Final ACC:', acc)
    print('Average loss:', loss)
    print('Total time:', (time_ed - time_st) * 1000)
    print('Average time:', (time_ed - time_st) / 10000 * 1000)
